import './stylesheets/userQuota.styl'; // used by both QuotaPoliciesWidget and ConfigView

import './routes';

// Extends and overrides API
import './models/CollectionModel';
import './models/UserModel';
import './views/CollectionView';
import './views/UploadWidget';
import './views/UserView';
